package gui;

import core.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

import java.io.*;
import javax.swing.filechooser.FileNameExtensionFilter;

public class MainFrame {

	private JFrame frm;

	private JButton open, write, convert, exit;

	public JTextField openFileTextField, writeFileTextField;

	private JPanel p1, p2, p3;

	private File f;

	private String ExcelFilePath, OKCFilePath;

	private JFileChooser fcOpen, fcWrite;

	private int flag, setFilter = -1;

	public MainFrame() {
		frm = new JFrame("Excel2Xmdv");

		open = new JButton("Select Excel File");
		openFileTextField = new JTextField(40);

		write = new JButton("Save As OKC File");
		writeFileTextField = new JTextField(40);

		convert = new JButton("Convert");
		exit = new JButton("Exit");

		p1 = new JPanel();
		p2 = new JPanel();
		p3 = new JPanel();
		fcOpen = new JFileChooser();
		fcWrite = new JFileChooser();

		Container c = frm.getContentPane();

		c.add(p1, BorderLayout.NORTH);
		c.add(p2, BorderLayout.CENTER);
		c.add(p3, BorderLayout.SOUTH);
		p1.add(open);
		p1.add(openFileTextField);
		p2.add(write);
		p2.add(writeFileTextField);
		p3.add(convert);
		p3.add(exit);

		// Add Button Event
		open.addActionListener(new action());
		write.addActionListener(new action());
		convert.addActionListener(new action());
		exit.addActionListener(new action());

		frm.setSize(800, 200);

		// open the frame in the the center of the screen
		frm.setLocationRelativeTo(null);

		frm.setVisible(true);

		// Set exit event
		frm.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	}

	/**
	 * Set .okc file name and location consistent with .xls file
	 * 
	 */
	private void setOKCFilePath() {

		int dotPos;

		dotPos = ExcelFilePath.lastIndexOf(".");

		OKCFilePath = ExcelFilePath.substring(0, dotPos) + ".okc";

	}

	/**
	 * Open an excel file
	 * 
	 */

	private void openFile() {

		fcOpen.setDialogTitle("Select Excel File or CSV File");

		try {

			// Set file extension filter as ".xls"
			FileNameExtensionFilter filter, filter2, filter3;

			if (setFilter == -1) {

				filter = new FileNameExtensionFilter(
						"Microsoft Office Excel   (*.xls)", "xls");

				filter2 = new FileNameExtensionFilter(
						"Comma Separated Values   (*.csv)", "csv");

				filter3 = new FileNameExtensionFilter(
						"All Acceptable File Types   (*.xls; *.csv)", "xls",
						"csv");

				fcOpen.addChoosableFileFilter(filter);
				fcOpen.addChoosableFileFilter(filter2);
				fcOpen.addChoosableFileFilter(filter3);

				fcOpen.setFileFilter(filter);

				setFilter = 1;
			}

			flag = fcOpen.showOpenDialog(frm);

			//			 Get opend file after button pressed
			if (flag == JFileChooser.APPROVE_OPTION) {

				f = fcOpen.getSelectedFile();
				ExcelFilePath = f.getPath();

				// Set open file path text
				openFileTextField.setText(f.getPath());

				System.out.println("open file----" + f.getName());
				setOKCFilePath();

				// Set saved .okc file name and path
				writeFileTextField.setText(OKCFilePath);

			} else {

			}

		} catch (HeadlessException head) {
			System.out.println("Open File Dialog ERROR!");
		}

	}

	/**
	 * Set .okc file name and path
	 * 
	 */

	private void writeFile() {

		fcWrite.setDialogTitle("Save OKC File");

		// Set file extension filter as ".okc"
		FileNameExtensionFilter filter = new FileNameExtensionFilter(
				"XmdvTool File Format   (*.okc)", "okc");
		fcWrite.setFileFilter(filter);

		try {
			flag = fcWrite.showSaveDialog(frm);
		} catch (HeadlessException he) {
			System.out.println("Save File Dialog ERROR!");
		}

		// Get saved file after button pressed
		if (flag == JFileChooser.APPROVE_OPTION) {

			f = fcWrite.getSelectedFile();

			writeFileTextField.setText(f.getPath());

		}
	}

	/**
	 * Convert excel file (.xls) to xmdvTool file (.okc)
	 * 
	 */

	private void convertFile() {

		// Get the file path and name from text field
		ExcelFilePath = openFileTextField.getText();
		OKCFilePath = writeFileTextField.getText();

		if (ExcelFilePath.equals("")) {
			JOptionPane.showMessageDialog(null, "No excel file selected",
					"Open File", JOptionPane.ERROR_MESSAGE);
			return;
		} else if (!ExcelFilePath.endsWith(".xls")
				&& !ExcelFilePath.endsWith("csv")) {
			JOptionPane.showMessageDialog(null, "Please select a '*.xls' file",
					"Open File", JOptionPane.ERROR_MESSAGE);
			return;
		} else if (OKCFilePath.equals("")) {
			JOptionPane.showMessageDialog(null,
					"Please set the saved OKC file path", "Save File",
					JOptionPane.ERROR_MESSAGE);
			return;
		} else if (!OKCFilePath.endsWith(".okc")) {
			JOptionPane.showMessageDialog(null,
					"Saved file should be OKC file type (end with '.okc')",
					"Save File", JOptionPane.ERROR_MESSAGE);
			return;
		} else {
			ConvertExcel2Xmdv e2x = new ConvertExcel2Xmdv(ExcelFilePath,
					OKCFilePath);
			// read contents from excel file
			int readStatus = 0;

			if (ExcelFilePath.endsWith(".xls")) {
				readStatus = e2x.readExcel();
			} else if (ExcelFilePath.endsWith(".csv")) {
				readStatus = e2x.readCSV();
			}

			if (readStatus == -1) {
				JOptionPane.showMessageDialog(null, "The file '"
						+ openFileTextField.getText() + "' could not be found",
						"Open File", JOptionPane.ERROR_MESSAGE);
				return;
			} else if (readStatus == -2) {
				JOptionPane.showMessageDialog(null,
						"Please select an excel file or csv file", "Open File",
						JOptionPane.ERROR_MESSAGE);
				return;

			} else if (readStatus == -3) {
				JOptionPane.showMessageDialog(null, "The content of file '"
						+ openFileTextField.getText()
						+ "' is incorrect or invalid. Position: ("+ e2x.errRow +", "+ e2x.errCol + ")", "Open File",
						JOptionPane.ERROR_MESSAGE);
				return;
			} else if (readStatus == -4) {
				JOptionPane.showMessageDialog(null,
						"No content in sheet 1 of file '"
								+ openFileTextField.getText() + "' ",
						"Open File", JOptionPane.ERROR_MESSAGE);
				return;
			} else if (readStatus == -5) {
				JOptionPane.showMessageDialog(null,
						"No content in file '"
								+ openFileTextField.getText() + "' ",
						"Open File", JOptionPane.ERROR_MESSAGE);
				return;
			}

			int writeStatus = 0;
			// write to okc file
			writeStatus = e2x.writeOKC();
			if(writeStatus == -1){
				JOptionPane.showMessageDialog(null, "Unknown error when writing file '" + writeFileTextField.getText()
						+ "' has been converted successfully", "",
						JOptionPane.INFORMATION_MESSAGE);
			}
			else{
				JOptionPane.showMessageDialog(null, "The file '" + writeFileTextField.getText()
						+ "' has been converted successfully", "",
						JOptionPane.INFORMATION_MESSAGE);
			}

		}
	}

	/**
	 * set exit action
	 * 
	 */
	private void exit() {
		frm.dispose();
		System.exit(0);
	}

	/**
	 * 
	 * @author guo implement inner ActionListener class
	 * 
	 */

	class action implements ActionListener {
		public void actionPerformed(ActionEvent e) {

			// decide which button is pressed and execute operation
			if (e.getSource() == open)
				openFile();
			else if (e.getSource() == write)
				writeFile();
			else if (e.getSource() == convert)
				convertFile();
			else if (e.getSource() == exit)
				exit();
		}
	}

}
